# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class AlertCreateParams(RequestOptions):
    alert_type: Literal[
        "credit_balance_threshold", "spend_threshold", "usage_threshold"
    ]
    """
    The type of alert to create.
    """
    credit_balance_threshold: NotRequired[
        "AlertCreateParamsCreditBalanceThreshold"
    ]
    """
    The configuration of the credit balance threshold.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    spend_threshold: NotRequired["AlertCreateParamsSpendThreshold"]
    """
    The configuration of the spend threshold. An event fires when the amount consumed exceeds the threshold, after all credits and discounts are applied but before tax is applied.
    """
    title: str
    """
    The title of the alert.
    """
    usage_threshold: NotRequired["AlertCreateParamsUsageThreshold"]
    """
    The configuration of the usage threshold.
    """


class AlertCreateParamsCreditBalanceThreshold(TypedDict):
    filters: NotRequired[List["AlertCreateParamsCreditBalanceThresholdFilter"]]
    """
    The filters allows limiting the scope of this credit balance alert. You must specify a customer filter at this time.
    """
    lte: "AlertCreateParamsCreditBalanceThresholdLte"
    """
    Defines at which value the alert will fire.
    """


class AlertCreateParamsCreditBalanceThresholdFilter(TypedDict):
    credit_grants: NotRequired[
        "AlertCreateParamsCreditBalanceThresholdFilterCreditGrants"
    ]
    """
    The credit grants for which to configure the credit balance alert.
    """
    customer: NotRequired[str]
    """
    Limit the scope to this credit balance alert only to this customer.
    """
    type: Literal["customer", "tenant"]
    """
    What type of filter is being applied to this credit balance alert.
    """


class AlertCreateParamsCreditBalanceThresholdFilterCreditGrants(TypedDict):
    applicability_config: "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfig"
    """
    The applicability configuration for this credit grants filter.
    """


class AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfig(
    TypedDict,
):
    scope: "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScope"
    """
    Specify the scope of this applicability config.
    """


class AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScope(
    TypedDict,
):
    billable_items: NotRequired[
        List[
            "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopeBillableItem"
        ]
    ]
    """
    A list of billable items that the credit grant can apply to. We currently only support metered billable items. Cannot be used in combination with `price_type` or `prices`.
    """
    price_type: NotRequired[Literal["metered"]]
    """
    The price type that credit grants can apply to. We currently only support the `metered` price type. Cannot be used in combination with `prices`.
    """
    prices: NotRequired[
        List[
            "AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopePrice"
        ]
    ]
    """
    A list of prices that the credit grant can apply to. We currently only support the `metered` prices. Cannot be used in combination with `price_type`.
    """


class AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopeBillableItem(
    TypedDict,
):
    id: str
    """
    The billable item ID this credit grant should apply to.
    """


class AlertCreateParamsCreditBalanceThresholdFilterCreditGrantsApplicabilityConfigScopePrice(
    TypedDict,
):
    id: str
    """
    The price ID this credit grant should apply to.
    """


class AlertCreateParamsCreditBalanceThresholdLte(TypedDict):
    balance_type: Literal["custom_pricing_unit", "monetary"]
    """
    Specify the type of this balance. We currently only support `monetary` billing credits.
    """
    custom_pricing_unit: NotRequired[
        "AlertCreateParamsCreditBalanceThresholdLteCustomPricingUnit"
    ]
    """
    The custom pricing unit amount.
    """
    monetary: NotRequired["AlertCreateParamsCreditBalanceThresholdLteMonetary"]
    """
    The monetary amount.
    """


class AlertCreateParamsCreditBalanceThresholdLteCustomPricingUnit(TypedDict):
    id: str
    """
    The ID of the custom pricing unit.
    """
    value: Decimal
    """
    A positive decimal string representing the amount of the custom pricing unit threshold.
    """


class AlertCreateParamsCreditBalanceThresholdLteMonetary(TypedDict):
    currency: str
    """
    Three-letter [ISO code for the currency](https://stripe.com/docs/currencies) of the `value` parameter.
    """
    value: int
    """
    An integer representing the amount of the threshold.
    """


class AlertCreateParamsSpendThreshold(TypedDict):
    aggregation_period: Literal["billing"]
    """
    Defines the period over which spend is aggregated.
    """
    filters: NotRequired["AlertCreateParamsSpendThresholdFilters"]
    """
    Filters to scope the spend calculation.
    """
    group_by: NotRequired[
        Literal["billing_cadence", "pricing_plan_subscription"]
    ]
    """
    Defines the granularity of spend aggregation. Defaults to `pricing_plan_subscription`.
    """
    gte: "AlertCreateParamsSpendThresholdGte"
    """
    Defines at which value the alert will fire.
    """


class AlertCreateParamsSpendThresholdFilters(TypedDict):
    billable_items: NotRequired[List[str]]
    """
    Filter by billable item IDs. Maximum of 20 billable items.
    """
    billing_cadence: NotRequired[str]
    """
    Filter by billing cadence ID.
    """
    pricing_plan: NotRequired[str]
    """
    Filter by pricing plan ID.
    """
    pricing_plan_subscription: NotRequired[str]
    """
    Filter by pricing plan subscription ID.
    """


class AlertCreateParamsSpendThresholdGte(TypedDict):
    amount: NotRequired["AlertCreateParamsSpendThresholdGteAmount"]
    """
    The monetary amount. Required when type is `amount`. The threshold is the total_before_tax, the amount consumed after all credits and discounts are applied, but before tax is applied.
    """
    custom_pricing_unit: NotRequired[
        "AlertCreateParamsSpendThresholdGteCustomPricingUnit"
    ]
    """
    The custom pricing unit amount. Required when type is `custom_pricing_unit`.
    """
    type: Literal["amount", "custom_pricing_unit"]
    """
    The type of the threshold amount.
    """


class AlertCreateParamsSpendThresholdGteAmount(TypedDict):
    currency: str
    """
    Three-letter [ISO code for the currency](https://stripe.com/docs/currencies) of the `value` parameter.
    """
    value: int
    """
    An integer representing the amount of the threshold.
    """


class AlertCreateParamsSpendThresholdGteCustomPricingUnit(TypedDict):
    id: str
    """
    The ID of the custom pricing unit.
    """
    value: Decimal
    """
    A positive decimal string representing the amount of the custom pricing unit threshold.
    """


class AlertCreateParamsUsageThreshold(TypedDict):
    filters: NotRequired[List["AlertCreateParamsUsageThresholdFilter"]]
    """
    The filters allows limiting the scope of this usage alert. You can only specify up to one filter at this time.
    """
    gte: int
    """
    Defines the threshold value that triggers the alert.
    """
    meter: str
    """
    The [Billing Meter](https://docs.stripe.com/api/billing/meter) ID whose usage is monitored.
    """
    recurrence: Literal["one_time"]
    """
    Defines how the alert will behave.
    """


class AlertCreateParamsUsageThresholdFilter(TypedDict):
    customer: NotRequired[str]
    """
    Limit the scope to this usage alert only to this customer.
    """
    type: Literal["customer"]
    """
    What type of filter is being applied to this usage alert.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._price import Price
    from stripe.params.product_catalog._trial_offer_create_params import (
        TrialOfferCreateParams,
    )


class TrialOffer(CreateableAPIResource["TrialOffer"]):
    """
    Trial offers let you define free or paid introductory pricing for a subscription item.
    A TrialOffer specifies the price to charge during the trial, how long the trial lasts
    (a fixed end timestamp or a number of billing intervals), and what price the subscription
    item transitions to when the trial ends. You attach a TrialOffer to a subscription item
    using `items[current_trial][trial_offer]` when creating or updating a subscription.
    """

    OBJECT_NAME: ClassVar[Literal["product_catalog.trial_offer"]] = (
        "product_catalog.trial_offer"
    )

    class Duration(StripeObject):
        class Relative(StripeObject):
            iterations: int
            """
            The number of iterations of the price's interval for this trial offer.
            """

        relative: Optional[Relative]
        type: Literal["relative", "timestamp"]
        """
        The type of trial offer duration.
        """
        _inner_class_types = {"relative": Relative}

    class EndBehavior(StripeObject):
        class Transition(StripeObject):
            price: ExpandableField["Price"]
            """
            The new price to use at the end of the trial offer period.
            """

        transition: Transition
        type: Literal["transition"]
        """
        The type of behavior when the trial offer ends.
        """
        _inner_class_types = {"transition": Transition}

    duration: Duration
    end_behavior: EndBehavior
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    name: Optional[str]
    """
    A brief, user-friendly name for the trial offer-for identification purposes.
    """
    object: Literal["product_catalog.trial_offer"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    price: ExpandableField["Price"]
    """
    The price during the trial offer.
    """

    @classmethod
    def create(
        cls, **params: Unpack["TrialOfferCreateParams"]
    ) -> "TrialOffer":
        """
        Creates a trial offer.
        """
        return cast(
            "TrialOffer",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["TrialOfferCreateParams"]
    ) -> "TrialOffer":
        """
        Creates a trial offer.
        """
        return cast(
            "TrialOffer",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    _inner_class_types = {"duration": Duration, "end_behavior": EndBehavior}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class TransactionEntry(StripeObject):
    """
    TransactionEntries represent individual money movements across different states within a Transaction.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.transaction_entry"]] = (
        "v2.money_management.transaction_entry"
    )

    class BalanceImpact(StripeObject):
        available: Amount
        """
        Impact to the available balance.
        """
        inbound_pending: Amount
        """
        Impact to the inbound_pending balance.
        """
        outbound_pending: Amount
        """
        Impact to the outbound_pending balance.
        """

    class TransactionDetails(StripeObject):
        class Flow(StripeObject):
            account: Optional[str]
            """
            If applicable, the connected account associated with this Transaction's flow.
            """
            adjustment: Optional[str]
            """
            If applicable, the ID of the Adjustment that created this Transaction.
            """
            application_fee: Optional[str]
            """
            If applicable, the ID of the Application Fee that created this Transaction.
            """
            application_fee_refund: Optional[str]
            """
            If applicable, the ID of the Application Fee Refund that created this Transaction.
            """
            charge: Optional[str]
            """
            If applicable, the ID of the Charge that created this Transaction.
            """
            currency_conversion: Optional[str]
            """
            In the future, this will be the ID of the currency conversion that created this Transaction. For now, this field is always null.
            """
            debit_dispute: Optional[str]
            """
            If applicable, the ID of the DebitDispute that created this Transaction.
            """
            dispute: Optional[str]
            """
            If applicable, the ID of the Dispute that created this Transaction.
            """
            fee_transaction: Optional[str]
            """
            If applicable, the ID of the FeeTransaction that created this Transaction.
            """
            inbound_transfer: Optional[str]
            """
            If applicable, the ID of the InboundTransfer that created this Transaction.
            """
            issuing_authorization: Optional[str]
            """
            If applicable, the ID of the Issuing authorization that created this Transaction.
            """
            issuing_dispute: Optional[str]
            """
            If applicable, the ID of the Issuing dispute that created this Transaction.
            """
            issuing_transaction: Optional[str]
            """
            If applicable, the ID of the Issuing transaction that created this Transaction.
            """
            outbound_payment: Optional[str]
            """
            If applicable, the ID of the OutboundPayment that created this Transaction.
            """
            outbound_transfer: Optional[str]
            """
            If applicable, the ID of the OutboundTransfer that created this Transaction.
            """
            payout: Optional[str]
            """
            If applicable, the ID of the Payout that created this Transaction.
            """
            received_credit: Optional[str]
            """
            If applicable, the ID of the ReceivedCredit that created this Transaction.
            """
            received_debit: Optional[str]
            """
            If applicable, the ID of the ReceivedDebit that created this Transaction.
            """
            refund: Optional[str]
            """
            If applicable, the ID of the Refund that created this Transaction.
            """
            reserve_hold: Optional[str]
            """
            If applicable, the ID of the Reserve Hold that created this Transaction.
            """
            reserve_release: Optional[str]
            """
            If applicable, the ID of the Reserve Release that created this Transaction.
            """
            tax_fund: Optional[str]
            """
            If applicable, the ID of the TaxFund that created this Transaction.
            """
            topup: Optional[str]
            """
            If applicable, the ID of the Topup that created this Transaction.
            """
            transfer: Optional[str]
            """
            If applicable, the ID of the Transfer that created this Transaction.
            """
            transfer_reversal: Optional[str]
            """
            If applicable, the ID of the Transfer Reversal that created this Transaction.
            """
            treasury_credit_reversal: Optional[str]
            """
            If applicable, the ID of the Treasury CreditReversal that created this Transaction.
            """
            treasury_debit_reversal: Optional[str]
            """
            If applicable, the ID of the Treasury DebitReversal that created this Transaction.
            """
            treasury_inbound_transfer: Optional[str]
            """
            If applicable, the ID of the Treasury InboundTransfer that created this Transaction.
            """
            treasury_issuing_authorization: Optional[str]
            """
            If applicable, the ID of the Treasury IssuingAuthorization that created this Transaction.
            """
            treasury_outbound_payment: Optional[str]
            """
            If applicable, the ID of the Treasury OutboundPayment that created this Transaction.
            """
            treasury_outbound_transfer: Optional[str]
            """
            If applicable, the ID of the Treasury OutboundTransfer that created this Transaction.
            """
            treasury_received_credit: Optional[str]
            """
            If applicable, the ID of the Treasury ReceivedCredit that created this Transaction.
            """
            treasury_received_debit: Optional[str]
            """
            If applicable, the ID of the Treasury ReceivedDebit that created this Transaction.
            """
            type: Literal[
                "adjustment",
                "application_fee",
                "application_fee_refund",
                "charge",
                "currency_conversion",
                "debit_dispute",
                "dispute",
                "fee_transaction",
                "inbound_transfer",
                "issuing_authorization",
                "issuing_dispute",
                "issuing_transaction",
                "outbound_payment",
                "outbound_transfer",
                "payout",
                "received_credit",
                "received_debit",
                "refund",
                "reserve_hold",
                "reserve_release",
                "tax_fund",
                "topup",
                "transfer",
                "transfer_reversal",
                "treasury_credit_reversal",
                "treasury_debit_reversal",
                "treasury_inbound_transfer",
                "treasury_issuing_authorization",
                "treasury_other",
                "treasury_outbound_payment",
                "treasury_outbound_transfer",
                "treasury_received_credit",
                "treasury_received_debit",
            ]
            """
            Open Enum. Type of the flow that created the Transaction. The field matching this value will contain the ID of the flow.
            """

        category: Literal[
            "adjustment",
            "advance",
            "anticipation_repayment",
            "balance_transfer",
            "climate_order_purchase",
            "climate_order_refund",
            "connect_collection_transfer",
            "connect_reserved_funds",
            "contribution",
            "currency_conversion",
            "debit_dispute",
            "dispute",
            "dispute_reversal",
            "financing_paydown",
            "financing_paydown_reversal",
            "inbound_payment",
            "inbound_payment_failure",
            "inbound_transfer",
            "inbound_transfer_reversal",
            "india_mdr_processing_fee",
            "issuing_authorization",
            "issuing_dispute",
            "issuing_dispute_fraud_liability_debit",
            "issuing_dispute_provisional_credit",
            "issuing_dispute_provisional_credit_reversal",
            "issuing_transaction",
            "minimum_balance_hold",
            "network_cost",
            "obligation",
            "outbound_payment",
            "outbound_payment_reversal",
            "outbound_transfer",
            "outbound_transfer_reversal",
            "partial_capture_reversal",
            "payment_method_passthrough_fee",
            "payment_network_reserved_funds",
            "platform_earning",
            "platform_earning_refund",
            "platform_fee",
            "platform_funded_credit_transaction",
            "received_credit",
            "received_credit_reversal",
            "received_debit",
            "received_debit_reversal",
            "refund",
            "refund_failure",
            "risk_reserved_funds",
            "stripe_balance_payment_debit",
            "stripe_balance_payment_debit_reversal",
            "stripe_fee",
            "stripe_fee_tax",
            "tax_fund",
            "tax_withholding",
            "transfer_reversal",
            "unreconciled_customer_funds",
        ]
        """
        Closed Enum for now, and will be turned into an Open Enum soon. A descriptive category used to classify the Transaction.
        """
        financial_account: str
        """
        Indicates the FinancialAccount affected by this Transaction.
        """
        flow: Optional[Flow]
        """
        Details about the Flow object that created the Transaction.
        """
        _inner_class_types = {"flow": Flow}

    balance_impact: BalanceImpact
    """
    The delta to the FinancialAccount's balance.
    """
    created: str
    """
    Time at which the object was created.
    """
    effective_at: str
    """
    Time at which the entry impacted (or will impact if it's in the future) the FinancialAccount balance.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.transaction_entry"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    transaction: str
    """
    The Transaction that this TransactionEntry belongs to.
    """
    transaction_details: TransactionDetails
    """
    Details copied from the transaction that this TransactionEntry belongs to.
    """
    treasury_transaction_entry: Optional[str]
    """
    The v1 Treasury transaction entry associated with this transaction entry.
    """
    _inner_class_types = {
        "balance_impact": BalanceImpact,
        "transaction_details": TransactionDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_create_params import (
        CustomPricingUnitOverageRateCreateParams as CustomPricingUnitOverageRateCreateParams,
    )
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_delete_params import (
        CustomPricingUnitOverageRateDeleteParams as CustomPricingUnitOverageRateDeleteParams,
    )
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_list_params import (
        CustomPricingUnitOverageRateListParams as CustomPricingUnitOverageRateListParams,
    )
    from stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_retrieve_params import (
        CustomPricingUnitOverageRateRetrieveParams as CustomPricingUnitOverageRateRetrieveParams,
    )
    from stripe.params.v2.billing.rate_cards._rate_create_params import (
        RateCreateParams as RateCreateParams,
        RateCreateParamsCustomPricingUnitAmount as RateCreateParamsCustomPricingUnitAmount,
        RateCreateParamsTier as RateCreateParamsTier,
        RateCreateParamsTransformQuantity as RateCreateParamsTransformQuantity,
    )
    from stripe.params.v2.billing.rate_cards._rate_delete_params import (
        RateDeleteParams as RateDeleteParams,
    )
    from stripe.params.v2.billing.rate_cards._rate_list_params import (
        RateListParams as RateListParams,
    )
    from stripe.params.v2.billing.rate_cards._rate_retrieve_params import (
        RateRetrieveParams as RateRetrieveParams,
    )
    from stripe.params.v2.billing.rate_cards._version_list_params import (
        VersionListParams as VersionListParams,
    )
    from stripe.params.v2.billing.rate_cards._version_retrieve_params import (
        VersionRetrieveParams as VersionRetrieveParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "CustomPricingUnitOverageRateCreateParams": (
        "stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_create_params",
        False,
    ),
    "CustomPricingUnitOverageRateDeleteParams": (
        "stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_delete_params",
        False,
    ),
    "CustomPricingUnitOverageRateListParams": (
        "stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_list_params",
        False,
    ),
    "CustomPricingUnitOverageRateRetrieveParams": (
        "stripe.params.v2.billing.rate_cards._custom_pricing_unit_overage_rate_retrieve_params",
        False,
    ),
    "RateCreateParams": (
        "stripe.params.v2.billing.rate_cards._rate_create_params",
        False,
    ),
    "RateCreateParamsCustomPricingUnitAmount": (
        "stripe.params.v2.billing.rate_cards._rate_create_params",
        False,
    ),
    "RateCreateParamsTier": (
        "stripe.params.v2.billing.rate_cards._rate_create_params",
        False,
    ),
    "RateCreateParamsTransformQuantity": (
        "stripe.params.v2.billing.rate_cards._rate_create_params",
        False,
    ),
    "RateDeleteParams": (
        "stripe.params.v2.billing.rate_cards._rate_delete_params",
        False,
    ),
    "RateListParams": (
        "stripe.params.v2.billing.rate_cards._rate_list_params",
        False,
    ),
    "RateRetrieveParams": (
        "stripe.params.v2.billing.rate_cards._rate_retrieve_params",
        False,
    ),
    "VersionListParams": (
        "stripe.params.v2.billing.rate_cards._version_list_params",
        False,
    ),
    "VersionRetrieveParams": (
        "stripe.params.v2.billing.rate_cards._version_retrieve_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.iam._api_key_create_params import ApiKeyCreateParams
    from stripe.params.v2.iam._api_key_expire_params import ApiKeyExpireParams
    from stripe.params.v2.iam._api_key_list_params import ApiKeyListParams
    from stripe.params.v2.iam._api_key_retrieve_params import (
        ApiKeyRetrieveParams,
    )
    from stripe.params.v2.iam._api_key_rotate_params import ApiKeyRotateParams
    from stripe.params.v2.iam._api_key_update_params import ApiKeyUpdateParams
    from stripe.v2._list_object import ListObject
    from stripe.v2.iam._api_key import ApiKey


class ApiKeyService(StripeService):
    def list(
        self,
        params: Optional["ApiKeyListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ApiKey]":
        """
        List all API Keys of an account.
        """
        return cast(
            "ListObject[ApiKey]",
            self._request(
                "get",
                "/v2/iam/api_keys",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ApiKeyListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ApiKey]":
        """
        List all API Keys of an account.
        """
        return cast(
            "ListObject[ApiKey]",
            await self._request_async(
                "get",
                "/v2/iam/api_keys",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ApiKeyCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Create an API Key. To create a secret key in livemode, a public key for encryption must be provided.
        """
        return cast(
            "ApiKey",
            self._request(
                "post",
                "/v2/iam/api_keys",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ApiKeyCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Create an API Key. To create a secret key in livemode, a public key for encryption must be provided.
        """
        return cast(
            "ApiKey",
            await self._request_async(
                "post",
                "/v2/iam/api_keys",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ApiKeyRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Retrieve an API Key. For livemode secret keys, secret tokens are only returned on creation, and never returned here.
        """
        return cast(
            "ApiKey",
            self._request(
                "get",
                "/v2/iam/api_keys/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ApiKeyRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Retrieve an API Key. For livemode secret keys, secret tokens are only returned on creation, and never returned here.
        """
        return cast(
            "ApiKey",
            await self._request_async(
                "get",
                "/v2/iam/api_keys/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["ApiKeyUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Update an API Key. Only parameters that are specified in the request will be updated.
        """
        return cast(
            "ApiKey",
            self._request(
                "post",
                "/v2/iam/api_keys/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["ApiKeyUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Update an API Key. Only parameters that are specified in the request will be updated.
        """
        return cast(
            "ApiKey",
            await self._request_async(
                "post",
                "/v2/iam/api_keys/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def expire(
        self,
        id: str,
        params: Optional["ApiKeyExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Expire an API Key. The specified key becomes invalid immediately.
        """
        return cast(
            "ApiKey",
            self._request(
                "post",
                "/v2/iam/api_keys/{id}/expire".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def expire_async(
        self,
        id: str,
        params: Optional["ApiKeyExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Expire an API Key. The specified key becomes invalid immediately.
        """
        return cast(
            "ApiKey",
            await self._request_async(
                "post",
                "/v2/iam/api_keys/{id}/expire".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def rotate(
        self,
        id: str,
        params: Optional["ApiKeyRotateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Rotate an API Key. A new key with the same properties is created and returned. The existing key is expired immediately, unless an expiry time is specified.
        """
        return cast(
            "ApiKey",
            self._request(
                "post",
                "/v2/iam/api_keys/{id}/rotate".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def rotate_async(
        self,
        id: str,
        params: Optional["ApiKeyRotateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApiKey":
        """
        Rotate an API Key. A new key with the same properties is created and returned. The existing key is expired immediately, unless an expiry time is specified.
        """
        return cast(
            "ApiKey",
            await self._request_async(
                "post",
                "/v2/iam/api_keys/{id}/rotate".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class GrantedTokenCreateParams(RequestOptions):
    customer: NotRequired[str]
    """
    The Customer that the SharedPaymentGrantedToken belongs to. Should match the Customer that the PaymentMethod is attached to if any.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    payment_method: str
    """
    The PaymentMethod that is going to be shared by the SharedPaymentGrantedToken.
    """
    shared_metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to the SharedPaymentGrantedToken.
    """
    usage_limits: "GrantedTokenCreateParamsUsageLimits"
    """
    Limits on how this SharedPaymentGrantedToken can be used.
    """


class GrantedTokenCreateParamsUsageLimits(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    expires_at: NotRequired[int]
    """
    Time at which this SharedPaymentToken expires and can no longer be used to confirm a PaymentIntent.
    """
    max_amount: int
    """
    Max amount that can be captured using this SharedPaymentToken
    """
    recurring_interval: NotRequired[Literal["month", "week", "year"]]
    """
    The recurring interval at which the shared payment token's amount usage restrictions reset.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import NotRequired


class MarginCreateParams(RequestOptions):
    active: NotRequired[bool]
    """
    Whether the margin can be applied to invoices, invoice items, or invoice line items or not. Defaults to `true`.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    name: NotRequired[str]
    """
    Name of the margin, which is displayed to customers, such as on invoices.
    """
    percent_off: float
    """
    Percent that will be taken off the subtotal before tax (after all other discounts and promotions) of any invoice to which the margin is applied.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class QueryRunRetrieveParams(TypedDict):
    include: NotRequired[List[Literal["result.file.schema"]]]
    """
    Any optional includes (see https://docs.stripe.com/api-includable-response-values).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ApiKeyRotateParams(TypedDict):
    expire_current_key_in_minutes: NotRequired[int]
    """
    Duration in minutes before the current key expires, with a maximum of 7 days (10080 minutes).
    If not provided, the current key expires immediately.
    """
    public_key: NotRequired["ApiKeyRotateParamsPublicKey"]
    """
    Public key for encrypting the new API key secret.
    This must a PEM-formatted RSA key suitable for encryption, >= 2048 bits.
    A public key is required when rotating secret keys.
    Publishable keys are not encrypted and a public key should not be included.
    """


class ApiKeyRotateParamsPublicKey(TypedDict):
    id: NotRequired[str]
    """
    Caller's identifier of the public key. This is used for tracking purposes only, and will be echoed in the response if provided.
    """
    pem_key: NotRequired["ApiKeyRotateParamsPublicKeyPemKey"]
    """
    PEM-formatted public key.
    """


class ApiKeyRotateParamsPublicKeyPemKey(TypedDict):
    algorithm: str
    """
    The encryption algorithm used with this key (e.g., RSA).
    """
    data: str
    """
    The PEM-encoded public key data. Newlines are required between header/footer and body, and optional within the body.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._deletable_api_resource import DeletableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._nested_resource_class_methods import nested_resource_class_methods
from stripe._search_result_object import SearchResultObject
from stripe._searchable_api_resource import SearchableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import (
    AsyncIterator,
    ClassVar,
    Iterator,
    List,
    Optional,
    cast,
    overload,
)
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._price import Price
    from stripe._product_feature import ProductFeature
    from stripe._tax_code import TaxCode
    from stripe.params._product_create_feature_params import (
        ProductCreateFeatureParams,
    )
    from stripe.params._product_create_params import ProductCreateParams
    from stripe.params._product_delete_feature_params import (
        ProductDeleteFeatureParams,
    )
    from stripe.params._product_delete_params import ProductDeleteParams
    from stripe.params._product_list_features_params import (
        ProductListFeaturesParams,
    )
    from stripe.params._product_list_params import ProductListParams
    from stripe.params._product_modify_params import ProductModifyParams
    from stripe.params._product_retrieve_feature_params import (
        ProductRetrieveFeatureParams,
    )
    from stripe.params._product_retrieve_params import ProductRetrieveParams
    from stripe.params._product_search_params import ProductSearchParams


@nested_resource_class_methods("feature")
class Product(
    CreateableAPIResource["Product"],
    DeletableAPIResource["Product"],
    ListableAPIResource["Product"],
    SearchableAPIResource["Product"],
    UpdateableAPIResource["Product"],
):
    """
    Products describe the specific goods or services you offer to your customers.
    For example, you might offer a Standard and Premium version of your goods or service; each version would be a separate Product.
    They can be used in conjunction with [Prices](https://api.stripe.com#prices) to configure pricing in Payment Links, Checkout, and Subscriptions.

    Related guides: [Set up a subscription](https://docs.stripe.com/billing/subscriptions/set-up-subscription),
    [share a Payment Link](https://docs.stripe.com/payment-links),
    [accept payments with Checkout](https://docs.stripe.com/payments/accept-a-payment#create-product-prices-upfront),
    and more about [Products and Prices](https://docs.stripe.com/products-prices/overview)
    """

    OBJECT_NAME: ClassVar[Literal["product"]] = "product"

    class Identifiers(StripeObject):
        ean: Optional[str]
        """
        European Article Number (EAN) consisting of 8 or 13 digits and optional dashes. You may optionally provide a leading 0 for a total of 14 digits. The final digit is a validated check digit.
        """
        gtin: Optional[str]
        """
        Global Trade Item Number (GTIN) consisting of 8, 12, 13, or 14 digits and optional dashes. The final digit is a validated check digit.
        """
        isbn: Optional[str]
        """
        International Standard Book Number (ISBN) consisting of 10 or 13 digits and optional dashes. The final digit is a validated check digit. For ISBN-10, the final digit may be a `X`.
        """
        jan: Optional[str]
        """
        Japanese Article Number (JAN) consisting of 13 digits and optional dashes. The first two digits must either be `45` or `49`. The final digit is a validated check digit.
        """
        mpn: Optional[str]
        """
        Manufacturer Part Number (MPN). May include up to 70 alphanumeric characters and dashes.
        """
        nsn: Optional[str]
        """
        National Stock Number (NSN) consisting of 13 digits and optional dashes. The seventh character may also be alphanumeric.
        """
        upc: Optional[str]
        """
        Universal Product Code (UPC) consisting of 12 digits and optional dashes. The final digit is a validated check digit.
        """

    class ManagedPayments(StripeObject):
        class IneligibilityReason(StripeObject):
            code: Optional[
                Literal["ineligible_tax_code", "no_tax_code_specified"]
            ]
            """
            A code identifying the reason this product can't be used with Managed Payments. Additional values might be added as Managed Payments evolves its eligibility criteria.
            """
            message: Optional[str]
            """
            A human-readable description of the reason this product can't be used with Managed Payments.
            """

        eligibility: Optional[Literal["eligible", "ineligible"]]
        """
        Whether this product is eligible for use with Managed Payments. Possible values are `eligible` and `ineligible`.
        """
        ineligibility_reasons: Optional[List[IneligibilityReason]]
        """
        The reasons this product is ineligible for use with Managed Payments, if any. This field isn't present if the product is eligible.
        """
        _inner_class_types = {"ineligibility_reasons": IneligibilityReason}

    class MarketingFeature(StripeObject):
        name: Optional[str]
        """
        The marketing feature name. Up to 80 characters long.
        """

    class PackageDimensions(StripeObject):
        height: float
        """
        Height, in inches.
        """
        length: float
        """
        Length, in inches.
        """
        weight: float
        """
        Weight, in ounces.
        """
        width: float
        """
        Width, in inches.
        """

    class TaxDetails(StripeObject):
        performance_location: Optional[str]
        """
        The performance location.
        """
        tax_code: Optional[str]
        """
        A [tax code](https://docs.stripe.com/tax/tax-categories) ID.
        """

    active: bool
    """
    Whether the product is currently available for purchase.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    default_price: Optional[ExpandableField["Price"]]
    """
    The ID of the [Price](https://docs.stripe.com/api/prices) object that is the default price for this product.
    """
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    description: Optional[str]
    """
    The product's description, meant to be displayable to the customer. Use this field to optionally store a long form explanation of the product being sold for your own rendering purposes.
    """
    id: str
    """
    Unique identifier for the object.
    """
    identifiers: Optional[Identifiers]
    images: List[str]
    """
    A list of up to 8 URLs of images for this product, meant to be displayable to the customer.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    managed_payments: Optional[ManagedPayments]
    marketing_features: List[MarketingFeature]
    """
    A list of up to 15 marketing features for this product. These are displayed in [pricing tables](https://docs.stripe.com/payments/checkout/pricing-table).
    """
    metadata: UntypedStripeObject[str]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: str
    """
    The product's name, meant to be displayable to the customer.
    """
    object: Literal["product"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    package_dimensions: Optional[PackageDimensions]
    """
    The dimensions of this product for shipping purposes.
    """
    shippable: Optional[bool]
    """
    Whether this product is shipped (i.e., physical goods).
    """
    statement_descriptor: Optional[str]
    """
    Extra information about a product which will appear on your customer's credit card statement. In the case that multiple products are billed at once, the first statement descriptor will be used. Only used for subscription payments.
    """
    tax_code: Optional[ExpandableField["TaxCode"]]
    """
    A [tax code](https://docs.stripe.com/tax/tax-categories) ID.
    """
    tax_details: Optional[TaxDetails]
    """
    Tax details for this product, including the [tax code](https://docs.stripe.com/tax/tax-codes) and an optional performance location.
    """
    type: Literal["good", "service"]
    """
    The type of the product. The product is either of type `good`, which is eligible for use with Orders and SKUs, or `service`, which is eligible for use with Subscriptions and Plans.
    """
    unit_label: Optional[str]
    """
    A label that represents units of this product. When set, this will be included in customers' receipts, invoices, Checkout, and the customer portal.
    """
    updated: int
    """
    Time at which the object was last updated. Measured in seconds since the Unix epoch.
    """
    url: Optional[str]
    """
    A URL of a publicly-accessible webpage for this product.
    """

    @classmethod
    def create(cls, **params: Unpack["ProductCreateParams"]) -> "Product":
        """
        Creates a new product object.
        """
        return cast(
            "Product",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["ProductCreateParams"]
    ) -> "Product":
        """
        Creates a new product object.
        """
        return cast(
            "Product",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_delete(
        cls, sid: str, **params: Unpack["ProductDeleteParams"]
    ) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "Product",
            cls._static_request(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def delete(sid: str, **params: Unpack["ProductDeleteParams"]) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        ...

    @overload
    def delete(self, **params: Unpack["ProductDeleteParams"]) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        ...

    @class_method_variant("_cls_delete")
    def delete(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["ProductDeleteParams"]
    ) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        return self._request_and_refresh(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    async def _cls_delete_async(
        cls, sid: str, **params: Unpack["ProductDeleteParams"]
    ) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "Product",
            await cls._static_request_async(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def delete_async(
        sid: str, **params: Unpack["ProductDeleteParams"]
    ) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        ...

    @overload
    async def delete_async(
        self, **params: Unpack["ProductDeleteParams"]
    ) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        ...

    @class_method_variant("_cls_delete_async")
    async def delete_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["ProductDeleteParams"]
    ) -> "Product":
        """
        Delete a product. Deleting a product is only possible if it has no prices associated with it. Additionally, deleting a product with type=good is only possible if it has no SKUs associated with it.
        """
        return await self._request_and_refresh_async(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    def list(
        cls, **params: Unpack["ProductListParams"]
    ) -> ListObject["Product"]:
        """
        Returns a list of your products. The products are returned sorted by creation date, with the most recently created products appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ProductListParams"]
    ) -> ListObject["Product"]:
        """
        Returns a list of your products. The products are returned sorted by creation date, with the most recently created products appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["ProductModifyParams"]
    ) -> "Product":
        """
        Updates the specific product by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Product",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["ProductModifyParams"]
    ) -> "Product":
        """
        Updates the specific product by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Product",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ProductRetrieveParams"]
    ) -> "Product":
        """
        Retrieves the details of an existing product. Supply the unique product ID from either a product creation request or the product list, and Stripe will return the corresponding product information.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ProductRetrieveParams"]
    ) -> "Product":
        """
        Retrieves the details of an existing product. Supply the unique product ID from either a product creation request or the product list, and Stripe will return the corresponding product information.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def search(
        cls, *args, **kwargs: Unpack["ProductSearchParams"]
    ) -> SearchResultObject["Product"]:
        """
        Search for products you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cls._search(search_url="/v1/products/search", *args, **kwargs)

    @classmethod
    async def search_async(
        cls, *args, **kwargs: Unpack["ProductSearchParams"]
    ) -> SearchResultObject["Product"]:
        """
        Search for products you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return await cls._search_async(
            search_url="/v1/products/search", *args, **kwargs
        )

    @classmethod
    def search_auto_paging_iter(
        cls, *args, **kwargs: Unpack["ProductSearchParams"]
    ) -> Iterator["Product"]:
        return cls.search(*args, **kwargs).auto_paging_iter()

    @classmethod
    async def search_auto_paging_iter_async(
        cls, *args, **kwargs: Unpack["ProductSearchParams"]
    ) -> AsyncIterator["Product"]:
        return (await cls.search_async(*args, **kwargs)).auto_paging_iter()

    @classmethod
    def delete_feature(
        cls,
        product: str,
        id: str,
        **params: Unpack["ProductDeleteFeatureParams"],
    ) -> "ProductFeature":
        """
        Deletes the feature attachment to a product
        """
        return cast(
            "ProductFeature",
            cls._static_request(
                "delete",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    async def delete_feature_async(
        cls,
        product: str,
        id: str,
        **params: Unpack["ProductDeleteFeatureParams"],
    ) -> "ProductFeature":
        """
        Deletes the feature attachment to a product
        """
        return cast(
            "ProductFeature",
            await cls._static_request_async(
                "delete",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve_feature(
        cls,
        product: str,
        id: str,
        **params: Unpack["ProductRetrieveFeatureParams"],
    ) -> "ProductFeature":
        """
        Retrieves a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            cls._static_request(
                "get",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    async def retrieve_feature_async(
        cls,
        product: str,
        id: str,
        **params: Unpack["ProductRetrieveFeatureParams"],
    ) -> "ProductFeature":
        """
        Retrieves a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            await cls._static_request_async(
                "get",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    def list_features(
        cls, product: str, **params: Unpack["ProductListFeaturesParams"]
    ) -> ListObject["ProductFeature"]:
        """
        Retrieve a list of features for a product
        """
        return cast(
            ListObject["ProductFeature"],
            cls._static_request(
                "get",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product)
                ),
                params=params,
            ),
        )

    @classmethod
    async def list_features_async(
        cls, product: str, **params: Unpack["ProductListFeaturesParams"]
    ) -> ListObject["ProductFeature"]:
        """
        Retrieve a list of features for a product
        """
        return cast(
            ListObject["ProductFeature"],
            await cls._static_request_async(
                "get",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product)
                ),
                params=params,
            ),
        )

    @classmethod
    def create_feature(
        cls, product: str, **params: Unpack["ProductCreateFeatureParams"]
    ) -> "ProductFeature":
        """
        Creates a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            cls._static_request(
                "post",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product)
                ),
                params=params,
            ),
        )

    @classmethod
    async def create_feature_async(
        cls, product: str, **params: Unpack["ProductCreateFeatureParams"]
    ) -> "ProductFeature":
        """
        Creates a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            await cls._static_request_async(
                "post",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product)
                ),
                params=params,
            ),
        )

    _inner_class_types = {
        "identifiers": Identifiers,
        "managed_payments": ManagedPayments,
        "marketing_features": MarketingFeature,
        "package_dimensions": PackageDimensions,
        "tax_details": TaxDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Any, Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class MetricQueryCreateParams(TypedDict):
    currency: NotRequired[str]
    """
    Which currency to return monetary metric results in.
    """
    ends_at: str
    """
    Timestamp denoting the end of the time range to request data for.
    """
    filters: NotRequired["Dict[str, Any]|UntypedStripeObject[Any]"]
    """
    Which dimension values to filter on; keys are dimension names, values are arrays of dimension values to filter on.
    """
    granularity: Literal["day", "month", "week", "year"]
    """
    The time granularity to aggregate results by.
    """
    group_by: NotRequired[List[str]]
    """
    Which dimension keys to group by; if not specified no grouping is performed.
    """
    limit: NotRequired[int]
    """
    The maximum number of rows in the response.
    """
    metrics: List["MetricQueryCreateParamsMetric"]
    """
    A list of the metrics to be returned; all metrics must share the same metric namespace.
    """
    page: NotRequired[str]
    """
    Pagination future-proofing: page token for navigating to next/previous batch of rows.
    """
    starts_at: str
    """
    Timestamp denoting the beginning of the time range to request data for.
    """
    timezone: NotRequired[str]
    """
    The timezone results should be in; defaults to the merchant's timezone.
    """


class MetricQueryCreateParamsMetric(TypedDict):
    id: NotRequired[str]
    """
    The ID for this metric, e.g. metric_61Sud3n5oAGVCWiSr5.
    """
    name: NotRequired[str]
    """
    The common name for this metric, e.g. revenue.mrr.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class ComponentListParams(TypedDict):
    limit: NotRequired[int]
    """
    Optionally set the maximum number of results per page. Defaults to 20.
    """
    lookup_keys: NotRequired[List[str]]
    """
    Filter by lookup keys. Mutually exclusive with `pricing_plan_version`.
    You can specify up to 10 lookup keys.
    """
    pricing_plan_version: NotRequired[str]
    """
    The ID of the Pricing Plan Version to list components for. Will use the latest version if not provided.
    Mutually exclusive with `lookup_keys`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ContractCancelParams(TypedDict):
    cancel_pricing_lines: NotRequired[
        List["ContractCancelParamsCancelPricingLine"]
    ]
    """
    Per-pricing-line proration behavior overrides. Falls back to `proration_behavior` if
    not specified for a given line.
    """
    include: NotRequired[
        List[
            Literal[
                "billing_settings",
                "one_time_fees",
                "pricing_lines",
                "pricing_overrides",
            ]
        ]
    ]
    """
    Additional fields to include in the response.
    """
    proration_behavior: NotRequired[Literal["none", "prorated"]]
    """
    Top-level proration behavior for the cancellation. Defaults to `prorated` if not set.
    """


class ContractCancelParamsCancelPricingLine(TypedDict):
    id: NotRequired[str]
    """
    The ID of the pricing line.
    """
    lookup_key: NotRequired[str]
    """
    The lookup key of the pricing line.
    """
    proration_behavior: NotRequired[Literal["none", "prorated"]]
    """
    Proration behavior scoped to this pricing line. If not provided, falls back to the
    top-level `proration_behavior` on the cancel request. Defaults to `prorated`.
    """
